function [aggrC] = depressionAnxietyAnls()

% Inputer variables - batchRun - 1 - Yes run in a batch mode with less
% required used input; 0 - No more user input is required. 

%     %% Constants
    h=[];
    TREES =1000; 
    % Set the maximum number of trees - The user can always choose a lower
    %number after the initial run for assessing the method's accuracy
    
    MAX_ITERS = 100; % This is a K-fold analysis with MAX_ITERS folds.
    TRAIN_PROP = 0.8; % What percentage is used for training (0.8 = 80%)
    
    % Batch run parameters - the code can be run in a batch mode, thus
    % without user's input - good for extenssive analysis with large number
    % of iterations and trees. These paramters are used for bath run.
    BATCH_No_GRPs = 2; %How many classes to segement the data 
    % What is the class of each group. '1 2 3', '4' is the classification
    % of groups 1,2 and 3 together with respect to group four. Yes/No -
    % Wheather to keep the group in the same size. If NO all data is used;
    % If YES - All groups' size is trimmed to the size of the smallest
    % group. The trimmed data is selected at random. 
    
    BATCH_GRPs = {'1 2 3', '4', 'Yes'}; 
    %BATCH_GRPs = {'1', '2', 'Yes'};
    
    options = statset('UseParallel',true);

%% Read Data
    [filename,pathname] = uigetfile('*.xlsx','Select the data file'); %INPUT FILE
    
    tableData = readtable(fullfile(pathname,filename));
    origData = tableData;
    partiID = table2array(tableData(:,'Subject'));
    tableData(:,'Subject') = [];
    
%% User Input No. of classes
    prompt = {'Enter the number of classes:'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans = {'4'};
    noGrps = str2num(cell2mat(inputdlg(prompt,dlg_title,num_lines,defaultans)));
    
    
    for grp=1:noGrps
        prompt{grp} = sprintf('group %d', grp);
        defaultans{grp} = sprintf('%d', grp);
    end
    prompt{grp+1} = 'Keep groups of the same size?';
    defaultans{grp+1} = 'No';
    
    dlg_title = 'Classes asignment';
    num_lines = 1;
    grps = inputdlg(prompt,dlg_title,num_lines,defaultans);
    
   tableDataByGroup = cell([length(grps)-1, 1]);
   clusterResults = cell([length(grps)-1, 1]);
   PartiIDByGroup = cell([length(grps)-1, 1]);
   TData = [];
   PartiID = [];
   clusterRes = [];
   
   for cluster = 1:(length(grps)-1)
        groupsInClass = str2num(grps{cluster});
        for grp = 1:length(groupsInClass)
            tableData_Inx = find(tableData.DASS_group == groupsInClass(grp));
            if (size(tableDataByGroup{cluster}) == 0)
                tableDataByGroup{cluster} = [tableData(tableData_Inx,:)];
                PartiIDByGroup{cluster} = [partiID(tableData_Inx)];
                clusterResults{cluster} = [cluster*ones(length(tableData_Inx),1)];
            else
                tableDataByGroup{cluster} = [tableDataByGroup{cluster}; tableData(tableData_Inx,:)];
                PartiIDByGroup{cluster} = [PartiIDByGroup{cluster}; partiID(tableData_Inx)];
                clusterResults{cluster} = [clusterResults{cluster}; cluster*ones(length(tableData_Inx),1)];
            end
            
        end
        clusterSize(cluster) = length(clusterResults{cluster});
   end

   if (strcmp(lower(grps{end}), 'yes'))
       maxNoObjectsInCluster = min(clusterSize);
       for cluster = 1:(length(clusterSize))
           clusterInx = randperm(clusterSize(cluster),maxNoObjectsInCluster);
           TData = [TData; tableDataByGroup{cluster}(clusterInx,:)];
           PartiID = [PartiID; PartiIDByGroup{cluster}(clusterInx,:)];
           clusterRes = [clusterRes; cluster*ones(maxNoObjectsInCluster,1)];
       end
   else
       for cluster = 1:(length(grps)-1)
           TData = [TData; tableDataByGroup{cluster}];
           PartiID = [PartiID; PartiIDByGroup{cluster}];
           clusterRes = [clusterRes; cluster*ones(clusterSize(cluster),1)];
       end
   end
           
           
   tableData = TData;
   DASSgroup = tableData(:,'DASS_group');
   tableData(:,'DASS_group') = [];
   
    %% Bagged Trees
    tableData(:,'BDI_SCORE') = [];
    tableData(:,'STAI_SCORE') = [];
    tableData(:,'PSWQ_SCORE') = [];
    tableData(:,'RRS_SCORE') = [];
    tableData(:,'DASS_Depression') = [];
    tableData(:,'DASS_Anxiety') = [];
    tableData(:,'DASS_stress') = [];

    VarsNames = strrep(tableData.Properties.VariableNames, '_', ' '); % Reading variables names
    rng(1945,'twister') % For reproducibility
    
    [noParticipants, noFeatures] = size(tableData);
    prompt = {'Enter the number of ITERATIONS:', 'Training Portions:', 'MAX No of Trees'};
    dlg_title = 'Input';
    num_lines = 1;
    defaultans{1} = num2str(MAX_ITERS);
    defaultans{2} = num2str(TRAIN_PROP);
    defaultans{3} = num2str(TREES);

    UInput = inputdlg(prompt,dlg_title,num_lines,defaultans);
    % Read user data for forest analysis
    noIters = str2num(UInput{1});
    trainPortion = str2num(UInput{2});
    noTrees = str2num(UInput{3});
    
    %% Finding features importance 
    Mdl = TreeBagger(noTrees,tableData, clusterRes , 'OOBPredictorImportance', 'on',...
            'Method','classification','OOBVarImp','On',...
            'Options',options);
    imp = Mdl.OOBPermutedPredictorDeltaError;
    [~,imp_sort]=sort(imp, 'descend'); %Get the order of B
    paramtersImportance = imp(imp_sort);
    varsNameImportance = Mdl.PredictorNames(imp_sort);
        
    figure(1);
    bar(paramtersImportance);
    ylabel('Predictor importance estimates');
    xlabel('Predictors');
    xticks(1:length(Mdl.PredictorNames));
    xticklabels(strrep(varsNameImportance,'_', '.'));
    xtickangle(45);
    grid on
    drawnow;
    title('Features sorted by importance');
    
    %% Taking only features that were found to be of importance, i.e., Importance > 0
    idxvar = find(imp>0);
    [noParticipants, ~] = size(tableData);
    
    Mdl = TreeBagger(noTrees, tableData(:,idxvar), clusterRes, 'OOBPredictorImportance', 'on', 'Method','classification', 'OOBVarImp','On', ...
        'Options',options);
    
    figure(2);
    plot(oobError(Mdl))
    xlabel('Number of Grown Trees')
    ylabel('Out-of-Bag Classification Error')
    drawnow;
    
    prompt = {'Enter the number of Trees:'};
    dlg_title = 'Input';
    num_lines = 1;
    minInx = min(find(oobError(Mdl) == min(oobError(Mdl))));
    defaultans = cellstr(num2str(minInx));
    noTrees = str2num(cell2mat(inputdlg(prompt,dlg_title,num_lines,defaultans)));
    
    aggrC = run_Iters(noIters, noTrees, tableData(:,idxvar), clusterRes, noGrps, noParticipants, trainPortion, PartiID, options);
    
end

function aggrC = run_Iters(ITERS, noTrees, tableData, clusterResults, noGrps, noParticipants, trainPortion, partiID, options)
    close all;
    subplot(2,2,1); colormap(gray(256));
    aggrC = zeros(noGrps);
    posCounts = zeros(1, size(tableData, 1));
    negCounts = zeros(1, size(tableData, 1));
    VarsNames = tableData.Properties.VariableNames;
    fullPartiAnals = zeros(noParticipants, ITERS);
    pairs= [];
    
    for iter = 1:ITERS
        randParticipants = randperm(noParticipants);
        trainParticipants = randParticipants(1:round(trainPortion*noParticipants));
        validParticipants = randParticipants(round((trainPortion)*noParticipants)+1:end);

        b5v = TreeBagger(noTrees,tableData(trainParticipants,:),clusterResults(trainParticipants), ...
            'OOBVarImp','off','OOBPred','on',...
            'Options',options);

        label = predict(b5v,tableData(validParticipants,:));
        [C,order] = confusionmat(clusterResults(validParticipants), str2num(cell2mat(label)), 'Order',[1:noGrps]);
        posInx = find(clusterResults(validParticipants) - str2num(cell2mat(label)) == 0);
        negInx = find(clusterResults(validParticipants) - str2num(cell2mat(label)) ~= 0);
        fullPartiAnals(validParticipants, iter) = str2num(cell2mat(label));

        posCounts(validParticipants(posInx)) = posCounts(validParticipants(posInx)) + 1;
        negCounts(validParticipants(negInx)) = negCounts(validParticipants(negInx)) + 1;

        subplot(2,2,2); stem(posCounts ./ (posCounts + negCounts)); axis tight;
        subplot(2,2,3); bar((posCounts + negCounts), 'r'); hold off; title ('Distribution of Sample Appearance');


        aggrC = aggrC + C;
        success(iter) = trace(aggrC)/sum(aggrC(:));

        grpsSum = sum(aggrC,2);
        for grp=1:noGrps
            C(grp,:) = aggrC(grp,:)/grpsSum(grp);
        end
        
        subplot(2,2,1); imagesc(C); title([num2str(iter) ' ' num2str(success(iter))]);
        subplot(2,2,4); plot(success); axis([1 ITERS 0 1]); drawnow

    end

    h = hist(fullPartiAnals', [0 1 2 3 4]);
    h(end+1,:) = clusterResults;
    h(end+1,:) = partiID';

    h(end+1,:) = posCounts ./ (posCounts + negCounts);
    countsRatio = posCounts ./ (posCounts + negCounts);

    % Analyze H by task
    % Find All 1 that were found as 1

    h = h';
    [lines, cl] = size(h);

    fprintf('0 \t1 \t2 \t3 \t4 \tTrue \t#\tSucc\n')
    for lin = 1:noParticipants
        for cla = 1:cl
            fprintf('%d \t', h(lin,cla));
        end
        fprintf('\n');

        maxInx(lin)= min(find(h(lin,2:5) == max(h(lin,2:5))));
        pairs(h(lin, 6), maxInx(lin), end+1) = find(partiID == h(lin, 7));
    end

    [szX, szY, szZ] = size(pairs);

    for randGen=1:20
        randClass = randi([1 noGrps], [1, noParticipants]);
        [h(randGen),pVal(randGen),e1(randGen),e2(randGen)] = testcholdout(maxInx, randClass, h(:,6),'Alternative','less');
        na = sum(randClass ~= h(:,6)' & (maxInx == h(:,6)'));
        nb = sum(maxInx ~= h(:,6)' & (randClass == h(:,6)'));
        m(randGen) = (abs(na - nb) - 1)^2/(na + nb);
    end

    fprintf('\n\n*** P-Value: %1.8f\n\n\n', mean(pVal));
         fprintf('\n\n*** Success Rate: %1.4f \t t=%1.2f\n', success(end), mean(m));
end
